import gurobi.GRB;
import gurobi.GRBConstr;
import gurobi.GRBEnv;
import gurobi.GRBException;
import gurobi.GRBLinExpr;
import gurobi.GRBModel;
import gurobi.GRBVar;

/**
 * Solver class. This class contains only static methods. Used to solve instances that already have a first stage solution x
 * @author Zhihao
 *
 */
class NDP {
	Network network;
	
	GRBModel model;				//the NDP model
	GRBVar[] y;					//flow vector. has length <number of arcs>
	GRBVar[] t;					//unmet demand vector. has length <number of demand nodes>
	GRBConstr[] capConstrs;		//capacity constraints. has length <number of arcs>
	GRBConstr[] demandConstrs;	//flow balance constraints for demand nodes. has length <number of demand nodes>
	
	/**
	 * Construct NDP with 0 capacity and 0 demand
	 */
	NDP(GRBEnv env,Network network_,double[] supplies) throws GRBException {
		network = network_;
		model = new GRBModel(env);
		 y = addYVars(model,network,0);
		 t = addTVars(model,network,0);
		capConstrs = addCapConstrsNoCap(model,y,network);
		demandConstrs = addFlowBalConstrsNoDemand(model,y,t,network,supplies,0);
		//OBJECTIVE
		GRBLinExpr obj = new GRBLinExpr();
		//flow cost
		for(int ij = 0; ij < network.arcs.length; ij++) {
			obj.addTerm(network.arcs[ij].flowCost,y[ij]);
		}
		//penalty
		for(int i = 0; i < network.demandNodes.length; i++) {
			obj.addTerm(network.nodes[network.demandNodes[i]].penalty,t[i]);
		}
		model.setObjective(obj,GRB.MINIMIZE);
	}
	/**
	 * Add flow variables
	 */
	static GRBVar[] addYVars(GRBModel model,Network network,int scenario) throws GRBException {
		GRBVar[] y = new GRBVar[network.arcs.length];
		for(int ij = 0; ij < network.arcs.length; ij++) {
			y[ij] = model.addVar(0,GRB.INFINITY,0,GRB.CONTINUOUS,"y("+scenario+","+network.arcs[ij].asString()+")");
		}
		model.update();
		return y;
	}
	/**
	 * Add unmet demand variables
	 */
	static GRBVar[] addTVars(GRBModel model,Network network,int scenario) throws GRBException {
		GRBVar[] t = new GRBVar[network.demandNodes.length];
		for(int i = 0; i < network.demandNodes.length; i++) {
			t[i] = model.addVar(0,GRB.INFINITY,0,GRB.CONTINUOUS,"t("+scenario+","+network.nodes[network.demandNodes[i]].name+")");
		}
		model.update();
		return t;
	}
	/**
	 * Adds capacity constraints
	 */
	static GRBConstr[] addCapConstrs(GRBModel model,GRBVar[] y,GRBVar[] x,Network network,int scenario) throws GRBException {
		GRBConstr[] capConstrs = new GRBConstr[network.arcs.length];
		GRBLinExpr temp;
		for(int ij = 0; ij < network.arcs.length; ij++) {
			temp = new GRBLinExpr();
			temp.addTerm(1,y[ij]);
			temp.addTerm(-1,x[ij]);
			capConstrs[ij] = model.addConstr(temp,GRB.LESS_EQUAL,0,"Capacity("+scenario+","+network.arcs[ij].asString()+")");
		}
		model.update();
		return capConstrs;
	}
	/**
	 * Adds capacity constraints with capacity 0
	 */
	private static GRBConstr[] addCapConstrsNoCap(GRBModel model,GRBVar[] y,Network network) throws GRBException {
		GRBConstr[] capConstrs = new GRBConstr[network.arcs.length];
		for(int ij = 0; ij < network.arcs.length; ij++) {
			capConstrs[ij] = model.addConstr(y[ij],GRB.LESS_EQUAL,0,"Capacity(0,"+network.arcs[ij].asString()+")");
		}
		model.update();
		return capConstrs;
	}
	/**
	 * Adds flow balance constraints
	 */
	static GRBConstr[] addFlowBalConstrs(GRBModel model,GRBVar[] y,GRBVar[] t,Network network,double[] supplies,double[] demands,int scenario) throws GRBException {
		GRBConstr[] demandConstrs = new GRBConstr[network.demandNodes.length];
		GRBLinExpr temp;
		for(int i = 0; i < network.supplyNodes.length; i++) {
			temp = flowBalLHS(network.nodes[network.supplyNodes[i]],y);
			model.addConstr(temp,GRB.LESS_EQUAL,supplies[i],"Flow balance("+network.nodes[network.supplyNodes[i]].name+")");
		}
		for(int i = 0; i < network.transitNodes.length; i++) {
			temp = flowBalLHS(network.nodes[network.transitNodes[i]],y);
			model.addConstr(temp,GRB.EQUAL,0,"Flow balance("+network.nodes[network.transitNodes[i]].name+")");
		}
		for(int i = 0; i < network.demandNodes.length; i++) {
			temp = flowBalLHS(network.nodes[network.demandNodes[i]],y);
			temp.addTerm(-1,t[i]);
			demandConstrs[i] = model.addConstr(temp,GRB.LESS_EQUAL,-demands[i],"Flow balance("+scenario+","+network.nodes[network.demandNodes[i]].name+")");
		}
		model.update();
		return demandConstrs;
	}
	/**
	 * Adds flow balance constraints with no demand
	 */
	private static GRBConstr[] addFlowBalConstrsNoDemand(GRBModel model,GRBVar[] y,GRBVar[] t,Network network,double[] supplies,int scenario) throws GRBException {
		double[] demands = new double[network.demandNodes.length];
		return addFlowBalConstrs(model,y,t,network,supplies,demands,scenario);
	}
	/**
	 * Generates the LHS of a flow balance constraint
	 */
	private static GRBLinExpr flowBalLHS(Node node,GRBVar[] y) {
		GRBLinExpr temp = new GRBLinExpr();
		for(int i = 0; i < node.arcsIn.size(); i++) {
			temp.addTerm(-1,y[node.arcsIn.get(i)]);
		}
		for(int i = 0; i < node.arcsOut.size(); i++) {
			temp.addTerm(1,y[node.arcsOut.get(i)]);
		}
		return temp;
	}
	/**
	 * Update model with an x vector
	 */
	void updateX(double[] x) throws GRBException {
		for(int ij = 0; ij < capConstrs.length; ij++) {
			capConstrs[ij].set(GRB.DoubleAttr.RHS,x[ij]);
		}
		model.update();
	}
	/**
	 * Update model with a demand vector
	 */
	void updateDemands(double[] demands) throws GRBException {
		for(int i = 0; i < demandConstrs.length; i++) {
			demandConstrs[i].set(GRB.DoubleAttr.RHS,-demands[i]);
		}
		model.update();
	}
}
