import gurobi.GRB;
import gurobi.GRBEnv;
import gurobi.GRBException;
import gurobi.GRBLinExpr;
import gurobi.GRBModel;
import gurobi.GRBVar;
/**
 * Solver class. This class contains only static methods. Used to solve instances using SNDP method
 * @author Zhihao
 *
 */
class SNDP {
	
	Network network;
	Instance instance;

	GRBModel model;
	GRBVar[] x;
	GRBVar[][] y;
	GRBVar[][] t;
	
	/**
	 * Constructs SNDP
	 */
	SNDP(GRBEnv env,Network network_,Instance instance_) throws GRBException {
		network = network_;
		instance = instance_;
		model = new GRBModel(env);
		x = addXVars(model,network,instance);
		y = new GRBVar[instance.scenarios.length][];
		t = new GRBVar[instance.scenarios.length][];
		for(int s = 0; s < instance.scenarios.length; s++) {
			 y[s] = NDP.addYVars(model,network,s);
			 t[s] = NDP.addTVars(model,network,s);
			 NDP.addCapConstrs(model,y[s],x,network,s);
			 NDP.addFlowBalConstrs(model,y[s],t[s],network,instance.supplies,instance.scenarios[s],s);
		}
		//OBJECTIVE
		GRBLinExpr obj = new GRBLinExpr();
		//capacity cost
		for(int ij = 0; ij < network.arcs.length; ij++) {
			obj.addTerm(network.arcs[ij].capCost,x[ij]);
		}
		//flow cost
		for(int s = 0; s < instance.scenarios.length; s++) {
			for(int ij = 0; ij < network.arcs.length; ij++) {
				obj.addTerm(network.arcs[ij].flowCost/instance.scenarios.length,y[s][ij]);
			}
		}
		//penalty
		for(int s = 0; s < instance.scenarios.length; s++) {
			for(int i = 0; i < network.demandNodes.length; i++) {
				obj.addTerm(network.nodes[network.demandNodes[i]].penalty/instance.scenarios.length,t[s][i]);
			}
		}
		model.setObjective(obj,GRB.MINIMIZE);
	}
	static GRBVar[] addXVars(GRBModel model,Network network,Instance instance) throws GRBException {
		GRBVar[] x = new GRBVar[network.arcs.length];
		for(int ij = 0; ij < network.arcs.length; ij++) {
			x[ij] = model.addVar(0,instance.totalDemandUB,0,GRB.CONTINUOUS,"x("+network.arcs[ij].asString()+")");
		}
		return x;
	}
	/**
	 *  Solves an instance using the SNDP approach
	 */
	Output solve() {
		try {
			Output output = new Output();
			long startTime = System.currentTimeMillis();
			model.optimize();
			//process solution
			output.solveTime = System.currentTimeMillis() - startTime;
			output.optVal = model.get(GRB.DoubleAttr.ObjVal);
			output.x = new double[x.length];
			for(int i = 0; i < x.length; i++) {
				output.x[i] = x[i].get(GRB.DoubleAttr.X);
			}
			output.ySol = new double[y.length][];
			for(int s = 0; s < y.length; s++) {
				output.ySol[s] = new double[y[s].length];
				for(int ij = 0; ij < y[s].length; ij++) {
					output.ySol[s][ij] = y[s][ij].get(GRB.DoubleAttr.X);
				}
			}
			output.tSol = new double[t.length][];
			for(int s = 0; s < t.length; s++) {
				output.tSol[s] = new double[t[s].length];
				for(int i = 0; i < t[s].length; i++) {
					output.tSol[s][i] = t[s][i].get(GRB.DoubleAttr.X);
				}
			}
			output.capCost = 0;
			for(int ij = 0; ij < network.arcs.length; ij++) {
				output.capCost += network.arcs[ij].capCost*output.x[ij];
			}
			output.secondStageSol = 0;
			output.flowCostSol = new double[instance.scenarios.length];
			output.penaltyCostSol = new double[instance.scenarios.length];
			for(int s = 0; s < instance.scenarios.length; s++) {
				for(int ij = 0; ij < network.arcs.length; ij++) {
					output.flowCostSol[s] += network.arcs[ij].flowCost*output.ySol[s][ij];
				}
				for(int i = 0; i < network.demandNodes.length; i++) {
					output.penaltyCostSol[s] += network.nodes[network.demandNodes[i]].penalty*output.tSol[s][i];
				}
				output.secondStageSol += output.flowCostSol[s]+output.penaltyCostSol[s];
			}
			output.secondStageSol /= instance.scenarios.length;
			return output;
		} catch (GRBException e) {
			System.err.println("Gurobi exception: " + e);
			return null;
		}
	}
}
