import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import org.apache.commons.math3.distribution.GammaDistribution;

import com.opencsv.CSVParser;
import com.opencsv.CSVParserBuilder;
import com.opencsv.CSVReader;
import com.opencsv.CSVReaderBuilder;
import com.opencsv.CSVWriter;

import org.apache.commons.io.FileUtils;

/**
 * Static helper class. Generates random samples 
 * @author Zhihao, Hideaki
 *
 */
public class SampleMaker {
	
	/**
	 * Randomly selects scenarios to form a sample and writes the sample to a file.
	 */
	public static void makeSamples(int numSupply, String mainDirName) {
		try {
			String inputDir = Main.mainDirectory + "/"+mainDirName+"/input files";
			Network network = ReaderWriter.file2network(inputDir+"/network.csv");
			Instance refset = ReaderWriter.file2instance(network,inputDir+"/reference set.csv");
			
			double trueTotalDemand = 0;
			for(int k = 0; k < refset.mu.length; k++) {
				trueTotalDemand += refset.mu[k];
			}
			
			//absolute lowest demand
			double[] scenarioSums = new double[refset.scenarios.length];
			for(int s = 0; s < refset.scenarios.length; s++) {
				for(int k = 0; k < refset.scenarios[s].length; k++) {
					scenarioSums[s] += refset.scenarios[s][k];
				}
			}
			int[] order = Sort.quicksort(scenarioSums);
			double[][] sample = new double[Main.sampleSize][];
			for(int m = 0; m < Main.sampleSize; m++) {
				sample[m] = refset.scenarios[order[m]];
			}
			double[] means = new double[refset.scenarios[0].length];
			double[] variances = new double[refset.scenarios[0].length];
			for(int k = 0; k < refset.scenarios[0].length; k++) {
				for(int j = 0; j < Main.sampleSize; j++) {
					means[k] += sample[j][k];
				}
				means[k] /= Main.sampleSize;
				for(int j = 0; j < Main.sampleSize; j++) {
					variances[k] += Math.pow(sample[j][k]-means[k],2);
				}
				variances[k] /= Main.sampleSize;
			}
			writeSample2File(numSupply,means,variances,sample,inputDir+"/sample 3");
			
			//added by Hideaki
			for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
					writeSAASample2File(numSupply,means,variances,inputDir+"/sample 3",Main.words[j],Main.sizes[k]);
				}
			}
			//

			double lowestTotalDemand = 0;
			for(int k = 0; k < refset.mu.length; k++) {
				lowestTotalDemand += means[k];
			}
			
			double[][] meanVar = null;
			//normal mean
			double singleVar;
			double maxSingleVar = Double.MAX_VALUE;
			while(maxSingleVar > Main.regularDemandParam) {
				sample = rollRandom(refset,Main.sampleSize);
				meanVar = getMeanVar(sample);
				//maximum deviation from the true demand at each node (want the deviation from true mean to be low)
				maxSingleVar = 0;
				for(int k = 0; k < meanVar[0].length; k++) {
					singleVar = Math.abs(meanVar[0][k]-refset.mu[k])/refset.mu[k];
					if(singleVar > maxSingleVar) {
						maxSingleVar = singleVar;
					}
				}
			}
			writeSample2File(numSupply,meanVar[0],meanVar[1],sample,inputDir+"/sample 1");

			//added by Hideaki
			for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
					writeSAASample2File(numSupply,meanVar[0],meanVar[1],inputDir+"/sample 1",Main.words[j],Main.sizes[k]);
				}
			}
			
			//low mean (total)
			double totalDemand = Double.MAX_VALUE;
			while((totalDemand-lowestTotalDemand)/(trueTotalDemand-lowestTotalDemand) > Main.lowDemandParam) {
				sample = rollRandom(refset,Main.sampleSize);
				meanVar = getMeanVar(sample);
				//ratio of total demand to the true total demand at each node (want the demand to be low)
				totalDemand = 0;
				for(int k = 0; k < meanVar[0].length; k++) {
					totalDemand += meanVar[0][k];
				}
			}
			writeSample2File(numSupply,meanVar[0],meanVar[1],sample,inputDir+"/sample 2");
			
			//added by Hideaki
			for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
					writeSAASample2File(numSupply,meanVar[0],meanVar[1],inputDir+"/sample 2",Main.words[j],Main.sizes[k]);
				}
			}
			//
				
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
	/**
	 * Randomly select a number of scenarios from the given reference set to form a sample
	 */
	public static double[][] rollRandom(Instance refset,int sampleSize) {
		double[][] output = new double[sampleSize][refset.scenarios[0].length];
		Random rng = new Random();
		for(int j = 0; j < sampleSize; j++) {
			output[j] = refset.scenarios[rng.nextInt(refset.scenarios.length)];
		}
		return output;
	}
	/**
	 * Get the means and variances of each randomly selected scenario
	 */
	public static double[][] getMeanVar(double[][] randomRoll) {
		double[][] output = new double[2][randomRoll[0].length];
		for(int k = 0; k < randomRoll[0].length; k++) {
			for(int j = 0; j < randomRoll.length; j++) {
				output[0][k] += randomRoll[j][k];
			}
			output[0][k] /= randomRoll.length;
			for(int j = 0; j < randomRoll.length; j++) {
				output[1][k] += Math.pow(randomRoll[j][k]-output[0][k],2);
			}
			output[1][k] /= randomRoll.length;
		}
		return output;
	}
	/**
	 * Write the sample to file
	 * @param means
	 * @param variances
	 * @param sample
	 * @param dirname
	 * @throws IOException
	 */
	public static void writeSample2File(int numSupply, double[] means,double[] variances,double[][] sample,String dirname) throws IOException {
		PrintWriter writer = new PrintWriter(dirname+"/drndp.csv");
		writer.println("Supplies:");
		for(int j = 0; j < numSupply; j++) {
			writer.printf(Main.supplyValue+",");
		}
		writer.printf("\n");
		writer.println();
		writer.println("Demand lower bounds:");
		for(int j = 0; j < means.length; j++) {
			writer.printf(Main.demandLB+",");
		}
		writer.printf("\n");
		writer.println();
		writer.println("Demand upper bounds:");
		for(int j = 0; j < means.length; j++) {
			writer.printf(Main.demandUB+",");
		}
		writer.printf("\n");
		writer.println();
		writer.println("Number of samples:");
		writer.println(sample.length);
		writer.println();
		writer.println("Demand samples:");
		for(int i = 0; i < sample.length; i++) {
			for(int j = 0; j < sample[i].length; j++) {
				writer.printf("%.0f,",sample[i][j]);
			}
			writer.printf("\n");
		}
		writer.close();
		writer = new PrintWriter(dirname+"/meanVar.csv");
		for(int j = 0; j < means.length; j++) {
			writer.printf(means[j]+",");
		}
		writer.printf("\n");
		for(int j = 0; j < variances.length; j++) {
			writer.printf(variances[j]+",");
		}
		writer.close();
	}
	/** added by Hideaki
	 * Write SAA samples to file
	 * @param means
	 * @param variances
	 * @param sample
	 * @param dirname
	 * @throws IOException
	 */
	public static void writeSAASample2File(int numSupply, double[] means,double[] variances,String dirname,String word,int size) throws IOException {
		double[][] sample = sampleSAA(word,size,means,variances);
		PrintWriter writer = new PrintWriter(dirname+"/"+word+size+".csv");
		writer.println("Supplies:");
		for(int j = 0; j < numSupply; j++) {
			writer.printf(Main.supplyValue+",");
		}
		writer.printf("\n");
		writer.println();
		writer.println("Demand lower bounds:");
		for(int j = 0; j < means.length; j++) {
			writer.printf(Main.demandLB+",");
		}
		writer.printf("\n");
		writer.println();
		writer.println("Demand upper bounds:");
		for(int j = 0; j < means.length; j++) {
			writer.printf(Main.demandUB+",");
		}
		writer.printf("\n");
		writer.println();
		writer.println("Number of samples:");
		writer.println(sample.length);
		writer.println();
		writer.println("Demand samples:");
		for(int i = 0; i < sample.length; i++) {
			for(int j = 0; j < sample[i].length; j++) {
				writer.printf("%.0f,",sample[i][j]);
			}
			writer.printf("\n");
		}
		writer.close();
	}
	/** added by Hideaki
	 * Sample gamma or uniform distribution
	 */
	private static double[][] sampleSAA(String word,int size,double[] means,double[] variances){
		double[][] sample = new double[size][means.length];
		switch (word) {
			case "gamma":
				for(int i = 0; i < size; i++) {
					for(int j = 0; j < means.length; j++) {
						double scale = variances[j]/means[j];
						double shape = means[j]/scale;
						double rnd = new GammaDistribution(shape,scale).sample();
						sample[i][j] = rnd;
					}
				}
				break;
			case "uniform":
				for(int i = 0; i < size; i++) {
					for(int j = 0; j < means.length; j++) {
						double upperPlusLower = means[j]*2;
						double upperMinusLower = Math.sqrt(variances[j]*12);
						double lower = (upperPlusLower-upperMinusLower)/2;
						sample[i][j] = lower + Math.random() * upperMinusLower;
					}
				}
				break;
			default:
				System.out.println("Invalid distribution");
				break;
		}
		return sample;
	}
	/** added by Hideaki
	 * parsing networks from SNDlib, and generate network and reference set files
	 */
	public static void networkParser(String filename,String dirname,int numSupply,int numDemand) {
		double supply = Main.supplyValue;
		double DLB = Main.demandLB;
		double DUB = Main.demandUB;
		int NOS = Main.numref;
		double aveCapCost = Main.aveCapCost;
		double sigCapCost = Main.sigCapCost;
		double aveFlowCost = Main.aveFlowCost;
		double sigFlowCost = Main.sigFlowCost;
		double PenaltyCost = Main.aveFlowCost; // make aveFlowCost == PenaltyCost
		double shape = Main.shape;
		double scale = Main.scale;
		
		try {
			List<String> nodeList = new ArrayList<String>();
			List<int[]> edgeList = new ArrayList<int[]>();
			
			BufferedReader br = new BufferedReader(new FileReader(filename));
			
			boolean lineString = true;
			while (lineString) {
				if (br.readLine().equals("NODES (")) {
					lineString = false;
				}
			}
			lineString = true;
			while (lineString) {
				String line = br.readLine();
				if (line.equals(")")) {
					lineString = false;
				} else {
					String[] splitln = line.split(" ");
					nodeList.add(splitln[2]);
				}
			}
			
			lineString = true;
			
			while (lineString) {
				if (br.readLine().equals("LINKS (")) {
					lineString = false;
				}
			}
			lineString = true;
			while (lineString) {
				String line = br.readLine();
				if (line.equals(")")) {
					lineString = false;
				} else {
					String[] splitln = line.split(" ");
					int one = nodeList.indexOf(splitln[4]);
					int two = nodeList.indexOf(splitln[5]);
					int[] arc1 = {one,two};
					int[] arc2 = {two,one};
					edgeList.add(arc1);
					edgeList.add(arc2);
				}
			}
			br.close();
			
			if (numSupply + numDemand > nodeList.size()) {
				throw new IllegalArgumentException("Invalid number of supplies and deamnds");
			}
			PenaltyCost = aveFlowCost * (nodeList.size()-1);
			List<Integer> nodes = new ArrayList<>();
			for (int i=0; i < nodeList.size(); i++) {
				nodes.add(i);
			}
			Collections.shuffle(nodes);
			
			PrintWriter writer = new PrintWriter(dirname+"/network.csv");
			writer.println("Number of nodes, supply nodes, demand nodes, arcs:");
			writer.println(nodeList.size()+","+numSupply+","+numDemand+","+edgeList.size()+"\n");
			writer.println("Nodes:");
			for (int i = 0; i < nodeList.size(); i++) {
				writer.println(i+"-"+nodeList.get(i));
			}
			writer.println("\nSupply node indices:");
			for(int i = 0; i < numSupply; i++) {
				writer.println(nodes.get(i));
			}
			writer.println("\nDemand node indices,penalties:");
			for(int i = 0; i < numDemand; i++) {
				writer.println(nodes.get(numSupply+i)+","+PenaltyCost);
			}
			Random r = new Random();
			writer.println("\nArcs, capacity costs, flow costs:");
			for (int i = 0; i < edgeList.size(); i++) {
				double capcost = r.nextGaussian()*sigCapCost + aveCapCost;
				capcost = (double)((int)(capcost*100))/100;
				double flowcost = r.nextGaussian()*sigFlowCost + aveFlowCost;
				flowcost = (double)((int)(flowcost*100))/100;
				writer.println(edgeList.get(i)[0]+","+edgeList.get(i)[1]+","+capcost+","+flowcost);
			}
			writer.close();
			
			writer = new PrintWriter(dirname+"/reference set.csv");
			writer.println("Supplies:");
			for (int i = 0; i<numSupply; i++) {
				writer.print(supply + ",");
			}
			writer.print("\n");
			writer.println("\nDemand lower bounds:");
			for (int i = 0; i<numDemand; i++) {
				writer.print(DLB + ",");
			}
			writer.print("\n");
			writer.println("\nDemand upper bounds:");
			for (int i = 0; i<numDemand; i++) {
				writer.print(DUB + ",");
			}
			writer.print("\n");
			writer.println("\nNumber of samples:");
			writer.println(NOS+"\n");
			writer.println("Scenarios");
			for(int i = 0; i < NOS; i++) {
				for (int j = 0; j < numDemand; j++) {
					double demand = 0;
					while (true) {
						double rnd = new GammaDistribution(shape,scale).sample();
						if (rnd < DUB) {
							demand = (double)((int)(rnd*100))/100;
							break;
						}
					}
					writer.print(demand);
					if (j < numDemand-1) {
						writer.print(",");
					}
				}
				writer.print("\n");
			}
			writer.close();
			
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
		
	}
	/** added by Hideaki
	 * generate grid network and reference set files
	 */
	public static void gridNetwork(String dirname,int gridsize,int numSupply,int numDemand) {
		try {
			double supply = Main.supplyValue;
			double DLB = Main.demandLB;
			double DUB = Main.demandUB;
			int NOS = Main.numref;
			double aveCapCost = Main.aveCapCost;
			double sigCapCost = Main.sigCapCost;
			double aveFlowCost = Main.aveFlowCost;
			double sigFlowCost = Main.sigFlowCost;
			double shape = Main.shape;
			double scale = Main.scale;
			int nodenum = gridsize * gridsize;
			double PenaltyCost = aveFlowCost * (nodenum-1); // PenaltyCostRule
		
			List<int[]> edgeList = new ArrayList<int[]>();
			for (int i=0; i < nodenum; i++) {
				if (i-gridsize >= 0) {
					int[] arc = {i,i-gridsize};
					edgeList.add(arc);
				}
				if (i%gridsize != 0) {
					int[] arc = {i,i-1};
					edgeList.add(arc);
				}
				if (i%gridsize != gridsize-1) {
					int[] arc = {i,i+1};
					edgeList.add(arc);
				}
				if (i+gridsize <= nodenum-1) {
					int[] arc = {i,i+gridsize};
					edgeList.add(arc);
				}
			}
		
			List<Integer> nodes = new ArrayList<>();
			for (int i=0; i < nodenum; i++) {
				nodes.add(i);
			}
			Collections.shuffle(nodes);
		
			PrintWriter writer = new PrintWriter(dirname+"/network.csv");
			writer.println("Number of nodes, supply nodes, demand nodes, arcs:");
			writer.println(nodenum+","+numSupply+","+numDemand+","+edgeList.size()+"\n");
			writer.println("Nodes:");
			for (int i = 0; i < nodenum; i++) {
				int name = i+1;
				writer.println(i+"-"+name);
			}
			writer.println("\nSupply node indices:");
			for(int i = 0; i < numSupply; i++) {
				writer.println(nodes.get(i));
			}
			writer.println("\nDemand node indices,penalties:");
			for(int i = 0; i < numDemand; i++) {
				writer.println(nodes.get(numSupply+i)+","+PenaltyCost);
			}
			writer.println("\nArcs, capacity costs, flow costs:");
			Random r = new Random();
			for (int i = 0; i < edgeList.size(); i++) {
				double capcost = r.nextGaussian()*sigCapCost + aveCapCost;
				capcost = (double)((int)(capcost*100))/100;
				double flowcost = r.nextGaussian()*sigFlowCost + aveFlowCost;
				flowcost = (double)((int)(flowcost*100))/100;
				writer.println(edgeList.get(i)[0]+","+edgeList.get(i)[1]+","+capcost+","+flowcost);
			}	
			writer.close();
			
			writer = new PrintWriter(dirname+"/reference set.csv");
			writer.println("Supplies:");
			for (int i = 0; i<numSupply; i++) {
				writer.print(supply + ",");
			}
			writer.print("\n");
			writer.println("\nDemand lower bounds:");
			for (int i = 0; i<numDemand; i++) {
				writer.print(DLB + ",");
			}
			writer.print("\n");
			writer.println("\nDemand upper bounds:");
			for (int i = 0; i<numDemand; i++) {
				writer.print(DUB + ",");
			}
			writer.print("\n");
			writer.println("\nNumber of samples:");
			writer.println(NOS+"\n");
			writer.println("Scenarios");
			for(int i = 0; i < NOS; i++) {
				for (int j = 0; j < numDemand; j++) {
					double demand = 0;
					while (true) {
						double rnd = new GammaDistribution(shape,scale).sample();
						if (rnd < DUB) {
							demand = (double)((int)(rnd*100))/100;
							break;
						}
					}
					writer.print(demand);
					if (j < numDemand-1) {
						writer.print(",");
					}
				}
				writer.print("\n");
			}
			writer.close();
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
	
	public static void copySamples(String sourceDirName,String mainDirName) {
		try {
			String sourceDir =sourceDirName + "/"+mainDirName+"/input files";
			String targetDir =Main.mainDirectory + "/"+mainDirName+"/input files";
			//sample 1
			NewFolder.create(targetDir+"/sample 1");
			File destDir1 = new File(targetDir+"/sample 1");
            File srcFile11 = new File(sourceDir+"/sample 1/drndp.csv");
            FileUtils.copyFileToDirectory(srcFile11, destDir1);
            
            File srcFile12 = new File(sourceDir+"/sample 1/meanVar.csv");
            FileUtils.copyFileToDirectory(srcFile12, destDir1);
            
            for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
		            File srcFile13 = new File(sourceDir+"/sample 1/" +Main.words[j] + Main.sizes[k] + ".csv");
		            FileUtils.copyFileToDirectory(srcFile13, destDir1);
				}
			}
            
			//sample 2
			NewFolder.create(targetDir+"/sample 2");
			File destDir2 = new File(targetDir+"/sample 2");
            File srcFile21 = new File(sourceDir+"/sample 2/drndp.csv");
            FileUtils.copyFileToDirectory(srcFile21, destDir2);
            
            File srcFile22 = new File(sourceDir+"/sample 2/meanVar.csv");
            FileUtils.copyFileToDirectory(srcFile22, destDir2);
            
            for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
		            File srcFile23 = new File(sourceDir+"/sample 2/" +Main.words[j] + Main.sizes[k] + ".csv");
		            FileUtils.copyFileToDirectory(srcFile23, destDir2);
				}
			}
            
			//sample 3
			NewFolder.create(targetDir+"/sample 3");
			File destDir3 = new File(targetDir+"/sample 3");
            File srcFile31 = new File(sourceDir+"/sample 3/drndp.csv");
            FileUtils.copyFileToDirectory(srcFile31, destDir3);
            
            File srcFile32 = new File(sourceDir+"/sample 3/meanVar.csv");
            FileUtils.copyFileToDirectory(srcFile32, destDir3);
            
            for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
		            File srcFile33 = new File(sourceDir+"/sample 3/" +Main.words[j] + Main.sizes[k] + ".csv");
		            FileUtils.copyFileToDirectory(srcFile33, destDir3);
				}
			}
				
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
	public static void copyInstance(String sourceDirName,String mainDirName) {
		try {
			NewFolder.create(Main.mainDirectory);
			NewFolder.create(Main.mainDirectory + "/"+mainDirName);
			String sourceDir =sourceDirName + "/"+mainDirName+"/input files";
			String targetDir =Main.mainDirectory + "/"+mainDirName+"/input files";
			
			NewFolder.create(targetDir);
			
			File networkfile = new File(sourceDir + "/network.csv");
			File referencefile = new File(sourceDir + "/reference set.csv");
			File newfolder = new File(targetDir);
			FileUtils.copyFileToDirectory(referencefile, newfolder);
			FileUtils.copyFileToDirectory(networkfile, newfolder);
			
			//sample 1
			NewFolder.create(targetDir+"/sample 1");
			File destDir1 = new File(targetDir+"/sample 1");
            File srcFile11 = new File(sourceDir+"/sample 1/drndp.csv");
            FileUtils.copyFileToDirectory(srcFile11, destDir1);
            
            File srcFile12 = new File(sourceDir+"/sample 1/meanVar.csv");
            FileUtils.copyFileToDirectory(srcFile12, destDir1);
            
            for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
		            File srcFile13 = new File(sourceDir+"/sample 1/" +Main.words[j] + Main.sizes[k] + ".csv");
		            FileUtils.copyFileToDirectory(srcFile13, destDir1);
				}
			}
            
			//sample 2
			NewFolder.create(targetDir+"/sample 2");
			File destDir2 = new File(targetDir+"/sample 2");
            File srcFile21 = new File(sourceDir+"/sample 2/drndp.csv");
            FileUtils.copyFileToDirectory(srcFile21, destDir2);
            
            File srcFile22 = new File(sourceDir+"/sample 2/meanVar.csv");
            FileUtils.copyFileToDirectory(srcFile22, destDir2);
            
            for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
		            File srcFile23 = new File(sourceDir+"/sample 2/" +Main.words[j] + Main.sizes[k] + ".csv");
		            FileUtils.copyFileToDirectory(srcFile23, destDir2);
				}
			}
            
			//sample 3
			NewFolder.create(targetDir+"/sample 3");
			File destDir3 = new File(targetDir+"/sample 3");
            File srcFile31 = new File(sourceDir+"/sample 3/drndp.csv");
            FileUtils.copyFileToDirectory(srcFile31, destDir3);
            
            File srcFile32 = new File(sourceDir+"/sample 3/meanVar.csv");
            FileUtils.copyFileToDirectory(srcFile32, destDir3);
            
            for(int j = 0; j < Main.words.length; j++) {
				for(int k = 0; k < Main.sizes.length; k++) {
		            File srcFile33 = new File(sourceDir+"/sample 3/" +Main.words[j] + Main.sizes[k] + ".csv");
		            FileUtils.copyFileToDirectory(srcFile33, destDir3);
				}
			}
				
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
	public static void changeLocation(String sourceDirName,String mainDirName) {
		try {
			NewFolder.create(Main.mainDirectory);
			NewFolder.create(Main.mainDirectory + "/"+mainDirName);
			
			String sourceDir =sourceDirName + "/"+mainDirName+"/input files";
			String targetDir =Main.mainDirectory + "/"+mainDirName+"/input files";
			NewFolder.create(targetDir);
			NewFolder.create(targetDir+"/sample 1");
			NewFolder.create(targetDir+"/sample 2");
			NewFolder.create(targetDir+"/sample 3");
			
			File referencefile = new File(sourceDir + "/reference set.csv");
			File newfolder = new File(targetDir);
			FileUtils.copyFileToDirectory(referencefile, newfolder);
			
			//CSVReader reader = new CSVReader(new FileReader(sourceDir + "/network.csv"), ',');
			CSVParser csvParser = new CSVParserBuilder().withSeparator(',').build();
			CSVReader reader = new CSVReaderBuilder(new FileReader(sourceDir + "/network.csv")).withCSVParser(csvParser).build();
			
		    List<String[]> myEntries = reader.readAll();
		    reader.close();
		    
			int nodenum = Integer.parseInt(myEntries.get(1)[0]);
			int supnum = Integer.parseInt(myEntries.get(1)[1]);
			int demnum = Integer.parseInt(myEntries.get(1)[2]);
			
			List<Integer> nodes = new ArrayList<>();
			for (int i=0; i < nodenum; i++) {
				nodes.add(i);
			}
			Collections.shuffle(nodes);
			
			
			for(int i = 0; i < supnum; i++) {
				myEntries.get(nodenum+6+i)[0] = nodes.get(i).toString();
			}
			for(int i = 0; i < demnum; i++) {
				myEntries.get(nodenum+supnum+8+i)[0] = nodes.get(supnum+i).toString();
			}
			
			CSVWriter writer = new CSVWriter(new FileWriter(targetDir+"/network.csv"),
					CSVWriter.DEFAULT_SEPARATOR,
					CSVWriter.NO_QUOTE_CHARACTER,
					CSVWriter.DEFAULT_ESCAPE_CHARACTER,
				    CSVWriter.DEFAULT_LINE_END);
			
			for(int i = 0; i < myEntries.size(); i++) {
				writer.writeNext(myEntries.get(i));
			}
			writer.close();
			
			
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
	
	public static void makeReferenceOnly(String sourceDirName,String mainDirName,int numSupply, int numDemand) {
		try {
			double supply = Main.supplyValue;
			double DLB = Main.demandLB;
			double DUB = Main.demandUB;
			int NOS = Main.numref;
			double shape = 12/numDemand;
			double scale = Main.scale;
			
			String sourceDir =sourceDirName + "/"+mainDirName+"/input files";
			String targetDir =Main.mainDirectory + "/"+mainDirName+"/input files";
			
			File networkfile = new File(sourceDir + "/network.csv");
			File newfolder = new File(targetDir);
			FileUtils.copyFileToDirectory(networkfile, newfolder);
			
			PrintWriter writer = new PrintWriter(targetDir+"/reference set.csv");
			writer.println("Supplies:");
			for (int i = 0; i<numSupply-1; i++) {
				writer.print(supply + ",");
			}
			writer.print(supply + "\n");
			writer.println("\nDemand lower bounds:");
			for (int i = 0; i<numDemand-1; i++) {
				writer.print(DLB + ",");
			}
			writer.print(DLB + "\n");
			writer.println("\nDemand upper bounds:");
			for (int i = 0; i<numDemand-1; i++) {
				writer.print(DUB + ",");
			}
			writer.print(DUB + "\n");
			writer.println("\nNumber of samples:");
			writer.println(NOS+"\n");
			writer.println("Scenarios");
			for(int i = 0; i < NOS; i++) {
				for (int j = 0; j < numDemand; j++) {
					double demand = 0;
					while (true) {
						double rnd = new GammaDistribution(shape,scale).sample();
						if (rnd < DUB) {
							demand = (double)((int)(rnd*100))/100;
							break;
						}
					}
					writer.print(demand);
					if (j < numDemand-1) {
						writer.print(",");
					}
				}
				writer.print("\n");
			}
			writer.close();
		} catch (IOException e) {
			System.err.println("I/O exception: " + e);
		}
	}
		
}
